"""Create messages for language models like Claude and OpenAI GPTs."""

# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/00_core.ipynb.

# %% auto 0
__all__ = ['mk_msg_openai', 'mk_msgs_openai', 'mk_msg', 'mk_msgs', 'Msg', 'AnthropicMsg', 'OpenAiMsg', 'mk_msg_anthropic',
           'mk_msgs_anthropic']

# %% ../nbs/00_core.ipynb
import base64
import mimetypes
from collections import abc

from fastcore import imghdr
from fastcore.meta import delegates
from fastcore.utils import *

# %% ../nbs/00_core.ipynb
def _mk_img(data:bytes)->tuple:
    "Convert image bytes to a base64 encoded image"
    img = base64.b64encode(data).decode("utf-8")
    mtype = mimetypes.types_map["."+imghdr.what(None, h=data)]
    return img, mtype

# %% ../nbs/00_core.ipynb
def mk_msg(content:Union[list,str], role:str="user", *args, api:str="openai", text_only=False, **kw)->dict:
    "Create an OpenAI/Anthropic compatible message."
    if text_only and not isinstance(content, str): raise("`content` muse be a string for `text_only` models.")
    m = OpenAiMsg if api == "openai" else AnthropicMsg
    return m()(role, content, text_only=text_only, **kw)

# %% ../nbs/00_core.ipynb
def mk_msgs(msgs: list, *args, api:str="openai", **kw) -> list:
    "Create a list of messages compatible with OpenAI/Anthropic."
    if isinstance(msgs, str): msgs = [msgs]
    return [mk_msg(o, ('user', 'assistant')[i % 2], *args, api=api, **kw) for i, o in enumerate(msgs)]

# %% ../nbs/00_core.ipynb
class Msg:
    "Helper class to create a message for the OpenAI and Anthropic APIs."
    def __call__(self, role:str, content:[list, str], text_only:bool=False, **kw)->dict:
        "Create an OpenAI/Anthropic compatible message with `role` and `content`."
        if hasattr(content, "content"): content, role = content.content, content.role
        content = self.find_block(content)
        if content is not None and not isinstance(content, list): content = [content]
        content = [self.mk_content(o, text_only=text_only) for o in content] if content else ''
        return dict(role=role, content=content[0] if text_only else content, **kw)

    def find_block(self, r)->dict:
        "Find the message in `r`."
        raise NotImplemented

    def text_msg(self, s:str, text_only:bool=False, **kw):
        "Convert `s` to a text message"
        return s if text_only else {"type":"text", "text":s}

    def img_msg(self, *args, **kw)->dict:
        "Convert bytes to an image message"
        raise NotImplemented

    def mk_content(self, content:[str, bytes], text_only:bool=False) -> dict:
        "Create the appropriate data structure based the content type."
        if isinstance(content, str): return self.text_msg(content, text_only=text_only)
        if isinstance(content, bytes): return self.img_msg(content)
        return content

# %% ../nbs/00_core.ipynb
class AnthropicMsg(Msg):
    def img_msg(self, data: bytes) -> dict:
        "Convert `data` to an image message"
        img, mtype = mk_img(data)
        r = {"type": "base64", "media_type": mtype, "data":img}
        return {"type": "image", "source": r}

    def find_block(self, r):
        "Find the message in `r`."
        return r.get('content', r) if isinstance(r, abc.Mapping) else r

# %% ../nbs/00_core.ipynb
class OpenAiMsg(Msg):
    def img_msg(self, data: bytes) -> dict:
        "Convert `data` to an image message"
        img, mtype = mk_img(data)
        r = {"url": f"data:{mtype};base64,{img}"}
        return {"type": "image_url", "image_url": r}

    def find_block(self, r):
        "Find the message in `r`."
        if type(r).__module__ == "builtins": return r
        m = nested_idx(r, "choices", 0)
        if not m: return m
        if hasattr(m, "message"): return m.message.content
        return m.delta

# %% ../nbs/00_core.ipynb
mk_msg_openai = partial(mk_msg, api="openai")
mk_msgs_openai = partial(mk_msgs, api="openai")

# %% ../nbs/00_core.ipynb
def _add_cache_control(msg, cache=False):
    "cache `msg`."
    if cache: msg["content"][-1]["cache_control"] = {"type": "ephemeral"}
    return msg

@delegates(mk_msg)
def mk_msg_anthropic(*args, cache=False, **kwargs):
    "Create an Anthropic compatible message."
    msg = partial(mk_msg, api="anthropic")(*args, **kwargs)
    return _add_cache_control(msg, cache=cache)

@delegates(mk_msgs)
def mk_msgs_anthropic(*args, cache=False, **kwargs):
    "Create a list of Anthropic compatible messages."
    msgs = partial(mk_msgs, api="anthropic")(*args, **kwargs)
    return [_add_cache_control(msg, cache=cache) for msg in msgs]
